@@ -1,18 +1,19 @@
 use std::{num::NonZeroU16, sync::Arc};
 
+use opentelemetry::trace::FutureExt;
 use pyo3::{pyclass, FromPyObject};
 use qcs::{Error, Executable, ExecutionData, JobHandle, Service};
 use rigetti_pyo3::{
-    impl_as_mut_for_wrapper, py_wrap_error, py_wrap_simple_enum, py_wrap_type,
+    impl_as_mut_for_wrapper, py_async, py_sync, py_wrap_error, py_wrap_simple_enum, py_wrap_type,
     pyo3::{exceptions::PyRuntimeError, pymethods, types::PyDict, Py, PyAny, PyResult, Python},
     wrap_error, PyWrapper, ToPython, ToPythonError,
 };
 use tokio::sync::Mutex;
+use tracing::instrument;
 
 use crate::{
     compiler::quilc::{PyCompilerOpts, PyQuilcClient},
     execution_data::PyExecutionData,
-    py_sync::{py_async, py_sync},
     qpu::{api::PyExecutionOptions, translation::PyTranslationOptions},
 };
 
@@ -67,7 +68,7 @@ macro_rules! py_executable_data {
                 .map(PyExecutionData::from)
                 .map_err(RustExecutionError::from)
                 .map_err(RustExecutionError::to_py_err)
-        }
+        }.with_current_context()
     }};
 }
 
@@ -86,9 +87,11 @@ macro_rules! py_job_handle {
                 .map_err(RustExecutionError::from)
                 .map_err(RustExecutionError::to_py_err)
         }
+        .with_current_context()
     }};
 }
 
+#[pyo3_opentelemetry::pypropagate(exclude(new), on_context_extraction_failure = "ignore")]
 #[pymethods]
 impl PyExecutable {
     #[new]
@@ -131,6 +134,7 @@ impl PyExecutable {
         Self::from(Arc::new(Mutex::new(exe)))
     }
 
+    #[instrument(skip_all)]
     pub fn execute_on_qvm(
         &self,
         py: Python<'_>,
@@ -139,6 +143,7 @@ impl PyExecutable {
         py_sync!(py, py_executable_data!(self, execute_on_qvm, &client))
     }
 
+    #[instrument(skip_all)]
     pub fn execute_on_qvm_async<'py>(
         &'py self,
         py: Python<'py>,
